/*
 * Copyright (c) 2018-2021. data2viz sàrl.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package io.data2viz.color

import io.data2viz.math.*

/**
 * Create a color in the LAB color space (CIE L*a*b* D65 whitepoint)
 *
 * @param lightness: Percent, value between 0% and 100%
 * @param aComponent "a"-component: Double for green-red between -128 and +128
 * @param bComponent "b"-component: Double for blue-yellow between -128 and +128
 * @param alpha: Opacity, value between 0% and 100%
 */
public class LabColor


@Deprecated("Deprecated", ReplaceWith("Colors.lab(labL,labA,labB,alpha)", "io.data2viz.colors.Colors"))
internal constructor(lightness: Percent, aComponent: Double, bComponent: Double, a: Percent = 100.pct) : Color {

    // TODO : need to choose behavior: if values are coerced the results are not what expected when referring to test values from chroma.js
    public val labL: Percent = lightness//.coerceIn(.0, 100.0)

    public val labA: Double = aComponent//.coerceIn(-128.0, 128.0)

    public val labB: Double = bComponent//.coerceIn(-128.0, 128.0)
    override val alpha: Percent = a.coerceToDefault()

    override val rgb: Int = toRgb().rgb
    override val rgba: String = toRgb().rgba
    override val r: Int = toRgb().r
    override val g: Int = toRgb().g
    override val b: Int = toRgb().b
    override val rgbHex: String = toRgb().rgbHex

    override fun luminance(): Percent = toRgb().luminance()
    override fun contrast(other:Color): Double = toRgb().contrast(other)

    override fun toRgb(): RgbColor = toRgba()
    override fun toLab(): LabColor = this
    override fun toHcl(): HclColor = toHcla()
    override fun toHsl(): HslColor = toRgb().toHsl()

    override fun brighten(strength: Double): Color = Colors.lab((labL + (Kn * strength).pct), labA, labB, alpha)
    override fun darken(strength: Double): Color = Colors.lab((labL - (Kn * strength).pct), labA, labB, alpha)
    override fun saturate(strength: Double): Color = toHcl().saturate(strength)
    override fun desaturate(strength: Double): Color = toHcl().desaturate(strength)
    override fun opacify(strength: Double): Color = withAlpha(Percent(alpha * strength))
    override fun withAlpha(alpha: Percent): LabColor = Colors.lab(labL, labA, labB, alpha)
    override fun withHue(hue: Angle): HclColor = toHcl().withHue(hue)

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other == null || other !is Color) return false

        if (rgb != other.rgb) return false
        if (alpha != other.alpha) return false

        return true
    }

    override fun hashCode(): Int {
        var result = rgb
        result = 31 * result + alpha.hashCode()
        return result
    }

    override fun toString(): String = "LAB($labL%, $labA, $labB, alpha=$alpha)"
}
